package com.bcxin.rest.web.apis.utils;

import org.apache.commons.codec.digest.DigestUtils;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * <b> 类CashierMD5Util.java的实现描述：MD5 工具类 </b>
 * @author ZXF
 * @create 2025/01/06 0006 18:34
 * @version
 * @注意事项 </b>
 */
public class CashierMD5Util {

    /**
     * @Title: buildRequestPara
     * @Description:
     * @param sParaTemp 没有签名的参数
     * @param key 私钥
     * @return Map<String,String>
     */
    public static Map<String, String> buildRequestPara(Map<String, String> sParaTemp, String key) {
        //除去数组中的空值和签名参数
        Map<String, String> sPara = paraFilter(sParaTemp);
        //生成签名结果
        String mysign = buildRequestMysign(sPara, key);

        //签名结果与签名方式加入请求提交参数组中
        sPara.put("sign", mysign);
        return sPara;
    }

    /**
     * @Title: buildRequestMysign
     * @author: ZXF
     * @Description: 生成签名结果
     * @param sPara
     * @param key
     * @return String 签名结果字符串
     */
    private static String buildRequestMysign(Map<String, String> sPara, String key) {
        String prestr = createLinkString(sPara); //把数组所有元素，按照“参数=参数值”的模式用“&”字符拼接成字符串
        System.out.println("参数排序："+prestr);
        String mysign = sign(prestr, key, "utf-8");
        return mysign;
    }

    /**
     * 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     *
     * @param params 需要排序并参与字符拼接的参数组
     * @return 拼接后字符串
     */
    public static String createLinkString(Map<String, String> params) {

        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);

        String prestr = "";

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key);

            if (i == keys.size() - 1) {//拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }

        return prestr;
    }


    /**
     * 除去数组中的空值和签名参数
     *
     * @param sArray 签名参数组
     * @return 去掉空值与签名参数后的新签名参数组
     */
    public static Map<String, String> paraFilter(Map<String, String> sArray) {

        Map<String, String> result = new HashMap<String, String>();

        if (sArray == null || sArray.size() <= 0) {
            return result;
        }

        for (String key : sArray.keySet()) {
            String value = sArray.get(key);
            if (value == null || value.equals("") || key.equalsIgnoreCase("sign") || key.equalsIgnoreCase("sign_type")) {
                continue;
            }
            result.put(key, value);
        }

        return result;
    }

    /**
     * 签名字符串
     *
     * @param text 需要签名的字符串
     * @param key 密钥
     * @param input_charset 编码格式
     * @return 签名结果
     */
    public static String sign(String text, String key, String input_charset) {
        text = text + key;
        System.out.println("参数排序01："+text);
        return DigestUtils.md5Hex(getContentBytes(text, input_charset));
    }

    /**
     * 签名字符串
     *
     * @param text 需要签名的字符串
     * @param sign 签名结果
     * @param key 密钥
     * @return 签名结果
     */
    public static boolean verify(String text, String sign, String key) {
        text = text + key;
        String mysign = DigestUtils.md5Hex(getContentBytes(text, "utf-8"));
        if (mysign.equals(sign)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @Title: verify
     * @param map
     * @param key
     * @return boolean
     */
    public static boolean verify(Map<String, String> map, String key) {
        String sign = "";
        if (map.get("sign") != null) {
            sign = map.get("sign");
        }
        String text = createLinkString(paraFilter(map));
        text = text + key;
        String mysign = DigestUtils.md5Hex(getContentBytes(text, "utf-8"));
        if (mysign.equals(sign)) {
            return true;
        } else {
            return false;
        }
    }

    public static boolean verify(Map<String, String> map, String sign, String key, String charset) {
        String text = createLinkString(paraFilter(map));
        text = text + key;
        String mysign = DigestUtils.md5Hex(getContentBytes(text, charset));
        if (mysign.equals(sign)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @Title: getContentBytes
     * @param content
     * @param charset
     * @return byte[]
     */
    private static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名过程中出现错误,指定的编码集不对,您目前指定的编码集是:" + charset);
        }
    }

    /**
     * @Title: verify
     * @param map
     * @param key
     * @return boolean
     */
    public static boolean verify(Map<String, String> map, String key,String signMsg) {
        String sign = "";
        if (map.get(signMsg) != null) {
            sign = map.get(signMsg);
        }
        String text = createLinkString(paraFilter(map,signMsg));
        text = text + key;
        String mysign = DigestUtils.md5Hex(getContentBytes(text, "utf-8"));
        if (mysign.equals(sign)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 除去数组中的空值和签名参数
     *
     * @param sArray 签名参数组
     * @return 去掉空值与签名参数后的新签名参数组
     */
    public static Map<String, String> paraFilter(Map<String, String> sArray,String signMsg) {

        Map<String, String> result = new HashMap<String, String>();

        if (sArray == null || sArray.size() <= 0) {
            return result;
        }

        for (String key : sArray.keySet()) {
            String value = sArray.get(key);
            if (value == null || value.equals("") || key.equalsIgnoreCase(signMsg)) {
                continue;
            }
            result.put(key, value);
        }

        return result;
    }
}
